from typing import Dict, Tuple

from ..assets import TableauRevampAsset

QUERY_TEMPLATE = """
{{
  {resource}Connection(first: {page_size}, after: AFTER_TOKEN_SIGNAL) {{
    nodes {{ {fields}
    }}
    pageInfo {{
      hasNextPage
      endCursor
    }}
    totalCount
  }}
}}
"""

_COLUMNS_QUERY = """
downstreamDashboards { id }
downstreamFields {
    id
    __typename
    datasource { id }
}
downstreamWorkbooks { id }
id
name
table { id }
"""

_DASHBOARDS_QUERY = """
createdAt
id
name
path
tags { name }
updatedAt
workbook { id }
"""

_DATASOURCES_QUERY = """
__typename
downstreamDashboards { id }
downstreamWorkbooks { id }
id
name
... on PublishedDatasource {
    description
    luid
    owner { luid }
    site { name }
    tags { name }
    uri
}
"""

_TABLES_QUERY = """
__typename
downstreamDashboards { id }
downstreamDatasources { id }
downstreamWorkbooks { id }
id
name
... on DatabaseTable {
    fullName
    schema
    database {
        connectionType
        id
        name
    }
}
... on CustomSQLTable {
    query
    database {
        connectionType
        id
        name
    }
}
"""


_WORKBOOKS_QUERY = """
createdAt
description
embeddedDatasources { id }
id
luid
name
owner { luid }
site { name }
tags { name }
updatedAt
uri
"""

_FIELDS_QUERY = """
__typename
datasource { id }
description
downstreamDashboards { id }
downstreamWorkbooks { id }
folderName
id
name
dataType
role
"""


_FIELDS_QUERY_WITH_COLUMNS = f"""
{_FIELDS_QUERY}
columns {{
    name
   table {{ name }}
}}
"""

_SHEETS_QUERY = """
containedInDashboards { id }
createdAt
id
index
name
updatedAt
upstreamFields { name }
workbook { id }
"""


GQL_QUERIES: Dict[TableauRevampAsset, Tuple[str, str]] = {
    TableauRevampAsset.COLUMN: ("columns", _COLUMNS_QUERY),
    TableauRevampAsset.DASHBOARD: ("dashboards", _DASHBOARDS_QUERY),
    TableauRevampAsset.DATASOURCE: ("datasources", _DATASOURCES_QUERY),
    TableauRevampAsset.SHEET: ("sheets", _SHEETS_QUERY),
    TableauRevampAsset.TABLE: ("tables", _TABLES_QUERY),
    TableauRevampAsset.WORKBOOK: ("workbooks", _WORKBOOKS_QUERY),
}

FIELDS_QUERIES = (
    ("binFields", _FIELDS_QUERY),
    ("calculatedFields", _FIELDS_QUERY),
    ("columnFields", _FIELDS_QUERY_WITH_COLUMNS),
    ("groupFields", _FIELDS_QUERY),
)



from unittest.mock import patch

import pytest
import requests

from .client import DomoClient, DomoCredentials
from .endpoints import EndpointFactory


class FakeResponse:
    """
    Helper class to fake few requests Response capabilities in the test below
    """

    def __init__(self, content: dict, status_code: int = 200):
        self.content = content
        self.status_code = status_code

    def json(self) -> dict:
        return self.content


@patch.object(DomoClient, "_get")
def test_client___get_element_with_error_status(mock_get):
    # init
    creds = DomoCredentials("bim", "bam", "boom", "lol")
    client = DomoClient(creds)
    any_endpoint = EndpointFactory("truc").datasets

    # case no error
    content = {"happy": "path"}
    mock_get.return_value = FakeResponse(content=content)
    assert client._get_element(any_endpoint) == content

    # case error not ignored, no response -> raised
    error = requests.exceptions.ConnectionError
    mock_get.side_effect = error
    with pytest.raises(error):
        client._get_element(any_endpoint)

    # case error not ignored, with response -> raised
    error = requests.exceptions.ConnectionError
    status_code = 403
    response = FakeResponse(content={}, status_code=status_code)
    mock_get.side_effect = error(response=response)
    with pytest.raises(error):
        client._get_element(any_endpoint)

    # case error is ignored -> not raised
    element = client._get_element(
        any_endpoint, ignore_error_codes=(status_code,)
    )
    assert element == dict()

    # case another error ignored -> raised
    with pytest.raises(error):
        another_status_code = status_code + 4
        client._get_element(
            any_endpoint, ignore_error_codes=(another_status_code,)
        )



from typing import List, Optional, Tuple

Link = Tuple[str, str]
TablesColumns = Tuple[List[dict], List[dict]]
Ostr = Optional[str]
TimestampedLink = Tuple[str, str, Ostr]

OTimestampedLink = Optional[TimestampedLink]
